@@ -13,4 +13,8 @@ class ConfigNotFound(Exception):
 class FoundMultipleConfigs(Exception):
     def __init__(self, msg):
         super().__init__(msg)
-        self.msg = msg
\ No newline at end of file
+        self.msg = msg
+
+
+class ConfigParamDoesNotExist(Exception):
+    pass
